@@ -62,6 +62,7 @@ struct _FRCommand
 	guint propCanModify : 1;
 	guint propAddCanUpdate : 1;
 	guint propAddCanReplace : 1;
+	guint propAddCanStoreFolders : 1;
 	guint propExtractCanAvoidOverwrite : 1;
 	guint propExtractCanSkipOlder : 1;
 	guint propExtractCanJunkPaths : 1;
@@ -92,7 +93,8 @@ struct _FRCommandClass
 
 	/*<virtual functions>*/
 
-	void        (*list)           (FRCommand     *comm);
+	void        (*list)           (FRCommand     *comm,
+				       const char    *password);
 
 	void        (*add)            (FRCommand     *comm,
 				       GList         *file_list,
@@ -151,7 +153,8 @@ void           fr_command_construct          (FRCommand     *comm,
 void           fr_command_set_filename       (FRCommand     *comm,
 					      const char    *filename);
 
-void           fr_command_list               (FRCommand     *comm);
+void           fr_command_list               (FRCommand     *comm,
+					      const char    *password);
 
 void           fr_command_add                (FRCommand     *comm,
 					      GList         *file_list,
